#!/usr/bin/env python

import os
import time
from optparse import OptionParser
import signal

COVERITY_PLACE = "/.autodirect/app/Coverity/cov-analysis-linux64-7.0.1/bin/"
COVERITY_CONFIGURE = COVERITY_PLACE + "/cov-configure --compiler /usr/bin/gcc --comptype gcc"
COVERITY_MAKE = COVERITY_PLACE +  "/cov-build --dir bin/coverity "
COVERITY_ANALYZE = COVERITY_PLACE + "/cov-analyze --dir "
COVERITY_SUBMIT = COVERITY_PLACE + "/cov-commit-defects --host 10.7.0.184 --stream ipxe --user admin --password 3tango -dir " 


ROM_SUFFIX = ".mrom"
MERGEROM = "./util/mergerom.pl"
ROM_PREFIX_FILE = "arch/i386/prefix/romprefix.S"
OLD_PREFIX_FILE = "arch/i386/prefix/romprefix.S.old"
TEMP_OUTPUT_FILE = "arch/i386/prefix/romprefix.S.new"
GENERAL_FILE = "config/general.h"
LATEST_PXE_DIR = "/mswg/release/exprom/pxe/FlexBoot/latest/"

CLP_DIRS = {
        "dell": "/mswg/release/exprom/clp/dell-m1000e/",
        "hp": "/mswg/release/exprom/clp/hp/"
        }


DEVICES = {
        "25408": "ConnectX-25408",
        "25418": "ConnectX-25418",
        "26418": "ConnectX2-26418",
        "26428": "ConnectX2-VPI",
        "26438": "ConnectX2-VPI-V",
        "25448": "ConnectX-25448",
        "26448": "ConnectX2-EN",
        "25458": "ConnectX-25458",
        "26458": "ConnectX2-26458",
        "26468": "ConnectX2-26468",
        "26478": "ConnectX2-26478",
        "4096": "ConnectX3-4096",
        "4097": "ConnectX3-4097",
        "4098": "ConnectX3-4098",
        "4099": "ConnectX3",
        "4100": "ConnectX3-4100",
        "4101": "ConnectX3-4101",
        "4102": "ConnectX3-4102",
        "4103": "ConnectX3-4103",
        "4104": "ConnectX3-4104",
        "4105": "ConnectX3-4105",
        "4106": "ConnectX3-4106",
        "4107": "ConnectX3-4107",
        "4108": "ConnectX3-4108",
        "4109": "ConnectX3-4109",
        "4110": "ConnectX3-4110",
        "4111": "ConnectX3-4111",
        "4112": "ConnectX3-4112",
        "4113": "ConnectIB"
        }

def openFile(fileName):
    try:
        sourceFile = open(fileName,'r')
    except:
        print "ERROR: Can't open file: %s" % fileName
        cleanup()
        exit(1)
    lines = sourceFile.readlines()
    sourceFile.close()
    return lines

def writeOutput(chunk, fileName):
    try:
        outputFile = open(fileName, 'w')
    except:
        print "ERROR: Can't open file: %s" % BIST_CONF_FILENAME
        cleanup()
        exit(1)
    outputFile.write(chunk)
    outputFile.close()
    return

def configureParser(optParser):
    optParser.add_option("-d", "--device", help="Device ID. e.g. 4099. (default=ALL)")
    optParser.add_option("-v", "--version", default="3.4.300", help="FlexBoot version")
    optParser.add_option("--release", action="store_true", default=False, help="Build release" )
    optParser.add_option("--fc", action="store_true", default=False, help="Enable flash configurable" )
    optParser.add_option("--clp", action="store_true", default=False, help="Add CLP merged ROMs" )
    optParser.add_option("--countlines", action="store_true", default=False, help="Count lines of code and exit" )
    optParser.add_option("-p", "--protocol", default="ALL", choices=["ALL", "IB", "EN", "VPI"], help="ALL/EN/IB/VPI (default=ALL)")
    optParser.add_option("-t", "--tag", help="The source GIT tag to build - only when using --git (default is the latest/current)")
    optParser.add_option("-g", "--git", help="The source GIT repository")
    optParser.add_option("--coverity", action="store_true", default=False, help="Build with Coverity code coverage footprint" )
    optParser.add_option("--debug", default="", help="Add debug flags (<file1>:<verbosity>,<file2>:<verbosity>,...)")


def checkOptions(options):
    if options.release and options.debug != "":
        print "Can't build release version with debug flags"
        return 1

    if options.device:
        if (not options.device == "ALL") and (not options.device in DEVICES):
            print "Device not found. Please enter a valid device ID or \"ALL\""
            print "Valid devices:"
            for dev in DEVICES.keys():
                print "ID: " + dev + " - Device: " + DEVICES[dev]
            return 1
    else:
        return 1

    if len(options.version.split(".")) != 3:
        print "Invalid version format. Must be A.B.XYZ (e.g. 3.4.160)"
        return 1

    return 0

def addSet(compilationSet, protocol, isRelease, isFC):
    if isRelease:
        compilationSet.append([protocol, False])
        compilationSet.append([protocol, True])
    elif isFC:
        compilationSet.append([protocol, True])
    else:
        compilationSet.append([protocol, False])
    return

def updateRomPrefix(protocol, deviceId, version):
    os.system("\cp -f " + ROM_PREFIX_FILE + "_ " + ROM_PREFIX_FILE)
    os.system("rm -f " + OLD_PREFIX_FILE)
    os.system("rm -f " + TEMP_OUTPUT_FILE)

    versionList = version.split(".")
    for subVersion in versionList:
        if len ( subVersion ) > 4:
            print "Error romversion %s" % version
            cleanup()
            exit(1)

    prefix = "driver_version:\n .align 16\n .long 0x73786c6d\n .long 0x3a6e6769\n .long 0x0010"

    # Preparing rest of ROM extension
    prefix = prefix + "%04x" % int ( versionList[0] )
    prefix = prefix + "\n .long 0x" +  "%04x" % int ( versionList[1] )
    prefix = prefix + "%04x" % int ( versionList[2] )
    prefix = prefix + "\n .long " + str ( hex ( int ( deviceId ) ) )
    # Adding port number
    prefix = prefix + "00"
    # Adding protocol type
    if protocol == "EN":
        prefix = prefix + "01"
    elif protocol == "VPI":
        prefix = prefix + "02"
    else:
        # IB
        prefix = prefix + "00"

    prefix = prefix + "\n\n"

    #open file
    fileLines = openFile(ROM_PREFIX_FILE)

    #create new file
    chunk = ""
    foundRomVersion = False
    for line in fileLines:
        if foundRomVersion == True:
            if line.find("/* ROM version added */") != -1:
                foundRomVersion = False
            else:
                continue
        if line.find("driver_version:") != -1:
            foundRomVersion = True
            chunk = "".join([chunk, prefix])
            continue

        chunk = "".join([chunk, line])

    #write file into temporary file
    writeOutput(chunk, TEMP_OUTPUT_FILE)

    #replace original file
    print "Replacing ROM file"
    rc = os.system("mv -f " + ROM_PREFIX_FILE + " " + OLD_PREFIX_FILE)
    if rc != 0:
        print "Error replacing file 1"
        cleanup()
        exit(rc)

    rc = os.system("mv -f " + TEMP_OUTPUT_FILE + " " + ROM_PREFIX_FILE)
    if rc != 0:
        print "Error replacing file 2"
        cleanup()
        exit(rc)

    return

def build(version, requestedDevices, protocol, isFC, outputDir, coverityEnabled, debugFlags):
    for dev in requestedDevices:
        print "Current build configuration: " + protocol + " ; FC=" + str(isFC) + " ; " + dev
        if (isFC and dev != "4099" and dev != "4103") or (dev == "4113" and protocol != "IB"):
            print "Unsupported combination. Skipping..."
            continue

        updateRomPrefix(protocol, dev, version)

        flags = ""
        # Add IB flag
        if protocol == "IB":
            flags += " -DCONNECTX_IB_ONLY"
        # Add EN flag
        if protocol == "EN":
            flags += " -DCONNECTX_ETH_ONLY"
        # Add FC flag
        if isFC:
            flags +=" -DFLASH_CONFIGURATION_CAP"
        
        if flags != "":
            flags = flags.strip()
            flags = "EXTRA_CFLAGS='" + flags + "'"

        # Add debug flag
        if debugFlags != "":
            debugFlags = "DEBUG='" + debugFlags + "'"

        cmd = "make -j bin/" + requestedDevices[dev] + ".mrom " + flags + " " + debugFlags + " > /dev/null"

        rc = 0
        if coverityEnabled:
            
            print cmd
            rc = os.system(COVERITY_MAKE + cmd)
        else:
            print cmd
            rc = os.system(cmd)
        
        if rc != 0:
            print "Build failed!"
            return rc

        # Copy output file to build directory
        fileName = "FlexBoot-" + version + "_" + protocol + "_"
        if isFC:
            fileName += "FC_"
        if debugFlags != "":
            fileName += "DEBUG_"
        fileName += dev + ".mrom"

        os.system("\cp -f bin/" + requestedDevices[dev] + ".mrom " + " " + outputDir + "/" + fileName)

        if coverityEnabled:
            rc = os.system(COVERITY_ANALYZE + "bin/coverity")
            if rc != 0:
                print "Coverity analyze error! Exiting..."
                return rc
            rc = os.system(COVERITY_SUBMIT + "bin/coverity")
            if rc != 0:
                print "Coverity submit error! Exiting..."
                return rc

    return 0

def merge(srcDir, destDir):
    if (os.path.isdir(srcDir)):
        rc = os.system("mkdir -p " + destDir)
        if rc != 0:
            print "Failed to create directory " + dir
            return rc

        for root, dirs, files in os.walk(srcDir):
            for file in files:
                if file.find('4099') != -1:
                    # Merge DELL CLP for ConnectX3
	            if file.find('EN') != -1:
		        clpFileName = '.'.join(os.path.realpath(CLP_DIRS["dell"] + "latest_4099.bin").split("/")[-1].split(".")[:-1])
                        outputMrom = '.'.join(file.split(".")[:-1]) + "_" + clpFileName + ".mrom"
                        cmd = MERGEROM + " " + root + "/" + file + " " + CLP_DIRS["dell"] + "latest_4099.bin > " + destDir + "/" + outputMrom
                        print cmd
                        rc = os.system(cmd)
                        if rc != 0:
                            print "Failed to create file"

                    # Merge HP CLP for ConnectX3/ConnectX3Pro/ConnectIB
                    clpFileName = '.'.join(os.path.realpath(CLP_DIRS["hp"] + "latest_4099.bin").split("/")[-1].split(".")[:-1])
                    outputMrom = '.'.join(file.split(".")[:-1]) + "_" + clpFileName + ".mrom"
                    cmd = MERGEROM + " " + root + "/" + file + " " + CLP_DIRS["hp"] + "latest_4099.bin > " + destDir + "/" + outputMrom
                    print cmd
                    rc = os.system(cmd)
                    if rc != 0:
                        print "Failed to create file"

                if file.find('4103') != -1:
                    clpFileName = '.'.join(os.path.realpath(CLP_DIRS["hp"] + "latest_4103.bin").split("/")[-1].split(".")[:-1])
                    outputMrom = '.'.join(file.split(".")[:-1]) + "_" + clpFileName + ".mrom"
                    cmd = MERGEROM + " " + root + "/" + file + " " + CLP_DIRS["hp"] + "latest_4103.bin > " + destDir + "/" + outputMrom
                    print cmd
                    rc = os.system(cmd)
                    if rc != 0:
                        print "Failed to create file"

                if file.find('4113') != -1:
                    clpFileName = '.'.join(os.path.realpath(CLP_DIRS["hp"] + "latest_4113.bin").split("/")[-1].split(".")[:-1])
                    outputMrom = '.'.join(file.split(".")[:-1]) + "_" + clpFileName + ".mrom"
                    cmd = MERGEROM + " " + root + "/" + file + " " + CLP_DIRS["hp"] + "latest_4113.bin > " + destDir + "/" + outputMrom
                    print cmd
                    rc = os.system(cmd)
                    if rc != 0:
                        print "Failed to create file"

    return 0

def mergeCLP(srcDir, destDir):
    print "Creating CLP Merged ROMs..."
    merge(srcDir + "/IB", destDir + "/IB")
    merge(srcDir + "/VPI", destDir + "/VPI")
    merge(srcDir + "/EN", destDir + "/EN")
    merge(srcDir + "/FC/IB", destDir + "/FC/IB")
    merge(srcDir + "/FC/VPI", destDir + "/FC/VPI")
    merge(srcDir + "/FC/EN", destDir + "/FC/EN")

######################################################################
# MAIN
######################################################################
def cleanup():
    # Move back the original files
    os.system("mv -f " + GENERAL_FILE + "_ " + GENERAL_FILE)
    os.system("mv -f " + ROM_PREFIX_FILE + "_ " + ROM_PREFIX_FILE)
    os.system("rm -f " + OLD_PREFIX_FILE)
    os.system("rm -f " + TEMP_OUTPUT_FILE)

def signal_handler(signal, frame):
    print "You pressed Ctrl+C!"
    cleanup()
    exit(1)

if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal_handler)
    optParser = OptionParser();
    configureParser(optParser)
    (options, args) = optParser.parse_args()
    if (len(args) > 0):
        optParser.print_help()
        exit(1)

    if options.countlines:
        print "/.autodirect/mtrswgwork/accounter_lines_of_code/bin/loc_count.py -P ipxe -T pxe -V 0.1 -p " + os.getcwd()
        rc = os.system ( "/.autodirect/mtrswgwork/accounter_lines_of_code/bin/loc_count.py -P ipxe -T pxe -V 0.1 -p " + os.getcwd())
        if rc != 0:
            print "Counter lines analyser error"
            exit(rc)
        exit(0)

    rc = checkOptions(options)
    if rc != 0:
        optParser.print_help()
        exit(1)

    if options.git:
        tmpDir = "./cloned_" + time.strftime("%d_%m_%Y") + "/"
        if (os.path.isdir(tmpDir)):
            print "Removing " + tmpDir
            os.system("rm -rf " + tmpDir)
        cloneRes = os.system("git clone " + options.git + " " + tmpDir)
        if (cloneRes != 0):
            print "Failed to clone git repository. Exiting... (Valid git? Any free space in " + os.getcwd() + ")"
            cleanup()
            exit(cloneRes)
        if (options.tag):
            cloneRes = os.system("cd " + tmpDir + " && pwd && git checkout " + options.tag + " && cd -")
            if cloneRes != 0:
                print "Failed to checkout to the requested tag. Removing Cloned directory " + tmpDir
                os.system("rm -rf " + tmpDir)
                cleanup()
                exit(cloneRes)
        os.chdir(tmpDir + "src/" )
        os.system("pwd")
    elif options.tag:
        print "Building from current directory. Ignoring tag..."
    
    ##################################################################
    ## From here we work on the current directory (local or cloned) ##
    ##################################################################

    if options.coverity:
        # Set coverity configure
        print "************     Coverity configuration Start    ************"
        rc = os.system (COVERITY_CONFIGURE)
        if rc != 0:
            print "Coverity configuration error! Exiting..."
            cleanup()
            exit(1)
        print "************     Coverity configuration End    ************\n"


    # Create a backup files
    os.system("\cp -f " + GENERAL_FILE + " " + GENERAL_FILE + "_")
    os.system("\cp -f " + ROM_PREFIX_FILE + " " + ROM_PREFIX_FILE + "_")


    # Set general.h with version
    newlines = []
    oldlines = openFile(GENERAL_FILE)
    for line in oldlines:
        newlines.append(line.replace('__BLD_VERSION__', '__BUILD_VERSION__ \"' + options.version + '\"'))
    writeOutput("".join(newlines), GENERAL_FILE)

    if (options.device == "ALL"):
        requestedDevices = DEVICES
    else:
        requestedDevices = {options.device: DEVICES[options.device]}

    # Add versions to compile: [PROTOCOL, IS_FLASH_CONFIGURABLE]
    compilationSet = []
    if (options.protocol == "ALL" or options.protocol == "IB"):
        #add UNDI protocol to compile
        addSet(compilationSet, "IB", options.release, options.fc)
    if (options.protocol == "ALL" or options.protocol == "VPI"):
        #add UNDI protocol to compile
        addSet(compilationSet, "VPI", options.release, options.fc)
    if (options.protocol == "ALL" or options.protocol == "EN"):
        #add UNDI protocol to compile
        addSet(compilationSet, "EN", options.release, options.fc)

    # Create destination directory
    buildDir = "./build_" + options.version
    if (os.path.isdir(buildDir)):
        print "Removing " + buildDir
        rc = os.system("rm -rf " + buildDir)
        if rc != 0:
            print "Failed to remove directory " + buildDir + ". Exiting..."
            cleanup()
            exit(rc);
    print "Creating output directory: " + buildDir
    rc = os.system("mkdir " + buildDir);
    if rc != 0:
        print "Failed to create directory " + buildDir + ". Exiting..."
        cleanup()
        exit(rc);

    # Build versions
    os.system("make clean");
    prevSubset = compilationSet[0];
    for subset in compilationSet:
        if prevSubset[0] != subset[0] or prevSubset[1] != subset[1]:
            os.system("make clean");
        outputDir = buildDir
        if subset[1]:
            # FC enabled
            outputDir += "/FC"
        outputDir += "/" + subset[0]
        if (not os.path.isdir(outputDir)):
            rc = os.system("mkdir -p " + outputDir);
            if rc != 0:
                print "Failed to create directory " + outputDir + ". Exiting..."
                cleanup()
                exit(rc);
        rc = build(options.version, requestedDevices, subset[0], subset[1], outputDir, options.coverity, options.debug)
        if rc != 0:
            cleanup()
            exit(rc);
        prevSubset = subset

    # Merge CLP
    if options.clp or options.release:
        clpDir = buildDir + "/CLP"
        if (not os.path.isdir(clpDir)):
            rc = os.system("mkdir " + clpDir);
            if rc != 0:
                print "Failed to create directory " + clpDir + ". Exiting..."
                cleanup()
                exit(rc);
        mergeCLP(buildDir, clpDir)

    cleanup()

    print "Done."
    exit(0)
