#!/usr/bin/env python

import os
import time
from optparse import OptionParser
from os import walk

DEVICES = {
        "25408": "ConnectX-25408",
        "25418": "ConnectX-25418",
        "26418": "ConnectX2-26418",
        "26428": "ConnectX2-VPI",
        "26438": "ConnectX2-VPI-V",
        "25448": "ConnectX-25448",
        "26448": "ConnectX2-EN",
        "25458": "ConnectX-25458",
        "26458": "ConnectX2-26458",
        "26468": "ConnectX2-26468",
        "26478": "ConnectX2-26478",
        "4096": "ConnectX3-4096",
        "4097": "ConnectX3-4097",
        "4098": "ConnectX3-4098",
        "4099": "ConnectX3",
        "4100": "ConnectX3-4100",
        "4101": "ConnectX3-4101",
        "4102": "ConnectX3-4102",
        "4103": "ConnectX3-4103",
        "4104": "ConnectX3-4104",
        "4105": "ConnectX3-4105",
        "4106": "ConnectX3-4106",
        "4107": "ConnectX3-4107",
        "4108": "ConnectX3-4108",
        "4109": "ConnectX3-4109",
        "4110": "ConnectX3-4110",
        "4111": "ConnectX3-4111",
        "4112": "ConnectX3-4112",
        "4113": "ConnectIB"
        }

def configureParser(optParser):
    optParser.add_option("-d", "--dir", default="/mswg/release/exprom/pxe/FlexBoot/latest/", help="Release directory (default=/mswg/release/exprom/pxe/latest/)")
    optParser.add_option("-p", "--pxe", help="PXE version")

def checkPXE(file, resStr, version):
    idx = resStr.find('type=PXE');
    if idx == -1:
        print "Incorrect PXE type!"
        return -1

    idx = resStr.find('version=' + version);
    if idx == -1:
        print "Incorrect PXE version!"
        return -2

    idx = resStr.find('devid=');
    if idx == -1:
        print "Incorrect device ID!"
        return -3

    idx = idx + 6
    idx_end = idx + 4
    if file.find(resStr[idx:idx_end].strip()) == -1:
        print "Incorrect device ID!"
        return -4


    idx = resStr.find('proto=');
    if idx == -1:
        print "Incorrect protocol!"
        return -5

    idx = idx + 6
    idx_end = idx + 3

    prot = resStr[idx:idx_end].strip()
    if prot == 'ETH':
        prot = 'EN'
    if file.find(prot) == -1:
        print "Incorrect protocol!"
        return -6

    return 0

def checkFile(file, version):
    if file.find('.mrom') == -1:
	print "Skipping file " + file
	return 0

    res = os.popen("flint -i " + file + " qrom")
    resStr = res.read()
    rc = checkPXE(file, resStr, version)

#    if file.find("CLP") != -1:
#       checkCLP(file)

    return rc

if __name__ == "__main__":
    optParser = OptionParser();
    configureParser(optParser)
    (options, args) = optParser.parse_args()
    if (len(args) > 0):
        optParser.print_help()
        exit(1)

    if not options.pxe:
        print "Please enter PXE version!"
	exit(1)

    print "Validating files in: " + options.dir
    
    fileList = []
    for (dirpath, dirnames, filenames) in walk(options.dir):
        for file in filenames:
            fileList.append(os.path.join(dirpath, file))

    errors = 0
    for file in fileList:
        rc = checkFile(file, options.pxe)
	if rc != 0:
	    errors = errors + 1
	    print "Error found (" + str(rc) + "): " + file

    if errors > 0:
        print "Totatl errors found " + str(errors) + "!"
	exit(1)

    print "No errors found. (Total files " + str(len(fileList)) + ")"
    exit(0)

